@@ -1,9 +1,9 @@
 from __future__ import annotations
 
-import datetime as dt
 from collections.abc import Iterable
 from io import StringIO
 from logging import DEBUG, WARNING, FileHandler, StreamHandler, getLogger
+from math import inf, isnan, nan
 from pathlib import Path
 from re import search
 from typing import TYPE_CHECKING, Any
@@ -12,7 +12,6 @@ from hypothesis import assume, given
 from hypothesis.strategies import (
     booleans,
     builds,
-    dates,
     dictionaries,
     integers,
     lists,
@@ -26,14 +25,14 @@ from polars import Object, String, UInt64
 from pytest import approx, mark, param, raises
 
 from tests.conftest import SKIPIF_CI_AND_WINDOWS
-from tests.test_operator import (
+from tests.test_objects.objects import (
     CustomError,
     SubFrozenSet,
     SubList,
     SubSet,
     SubTuple,
     TruthEnum,
-    make_objects,
+    objects,
 )
 from tests.test_typing_funcs.with_future import (
     DataClassFutureCustomEquality,
@@ -48,11 +47,12 @@ from tests.test_typing_funcs.with_future import (
     DataClassFutureNestedOuterFirstInner,
     DataClassFutureNestedOuterFirstOuter,
     DataClassFutureNone,
+    DataClassFutureTypeLiteral,
+    DataClassFutureTypeLiteralNullable,
 )
-from utilities.datetime import MINUTE, SECOND, get_now, get_now_local
 from utilities.functions import is_sequence_of
 from utilities.hypothesis import (
-    assume_does_not_raise,
+    dates,
     int64s,
     paths,
     temp_paths,
@@ -63,7 +63,7 @@ from utilities.hypothesis import (
 from utilities.iterables import always_iterable, one
 from utilities.logging import get_logging_level_number
 from utilities.math import MAX_INT64, MIN_INT64
-from utilities.operator import IsEqualError, is_equal
+from utilities.operator import is_equal
 from utilities.orjson import (
     _LOG_RECORD_DEFAULT_ATTRS,
     OrjsonFormatter,
@@ -75,17 +75,22 @@ from utilities.orjson import (
     _SerializeIntegerError,
     deserialize,
     get_log_records,
+    read_object,
     serialize,
+    write_object,
 )
 from utilities.polars import check_polars_dataframe, zoned_datetime
 from utilities.sentinel import Sentinel, sentinel
-from utilities.types import DateOrDateTime, LogLevel, MaybeIterable, PathLike
+from utilities.types import LogLevel, MaybeIterable, PathLike
 from utilities.typing import get_args
-from utilities.zoneinfo import UTC
+from utilities.tzlocal import LOCAL_TIME_ZONE
+from utilities.whenever import MINUTE, SECOND, get_now
 
 if TYPE_CHECKING:
     from collections.abc import Sequence
 
+    from whenever import Date, ZonedDateTime
+
     from utilities.types import Dataclass, StrMapping
 
 
@@ -123,11 +128,13 @@ class TestGetLogRecords:
         assert record.name == str(tmp_path)
         assert record.message == ""
         assert record.level == WARNING
-        assert record.line_num == approx(92, rel=0.1)
+        assert record.line_num == approx(102, rel=0.1)
         assert abs(record.datetime - get_now()) <= MINUTE
         assert record.func_name == "test_main"
         assert record.stack_info is None
-        assert record.extra == {"a": 1, "b": 2}
+        assert record.extra is not None
+        assert record.extra["a"] == 1
+        assert record.extra["b"] == 2
         assert record.log_file == file
         assert record.log_file_line_num == 1
 
@@ -167,7 +174,7 @@ class TestGetLogRecords:
                 "level": UInt64,
                 "path_name": String,
                 "line_num": UInt64,
-                "datetime": zoned_datetime(time_zone="local"),
+                "datetime": zoned_datetime(time_zone=LOCAL_TIME_ZONE),
                 "func_name": String,
                 "stack_info": String,
                 "extra": Object,
@@ -181,7 +188,7 @@ class TestGetLogRecords:
             tuples(
                 sampled_from(get_args(LogLevel)),
                 text_ascii(),
-                dictionaries(text_ascii(), integers()),
+                dictionaries(text_ascii(), int64s()),
             )
         ),
         root=temp_paths(),
@@ -193,9 +200,12 @@ class TestGetLogRecords:
         level=sampled_from(get_args(LogLevel)) | none(),
         min_level=sampled_from(get_args(LogLevel)) | none(),
         max_level=sampled_from(get_args(LogLevel)) | none(),
-        date_or_datetime=dates() | zoned_datetimes() | none(),
-        min_date_or_datetime=dates() | zoned_datetimes() | none(),
-        max_date_or_datetime=dates() | zoned_datetimes() | none(),
+        date=dates() | none(),
+        min_date=dates() | none(),
+        max_date=dates() | none(),
+        datetime=zoned_datetimes() | none(),
+        min_datetime=zoned_datetimes() | none(),
+        max_datetime=zoned_datetimes() | none(),
         func_name=booleans() | text_ascii() | none(),
         extra=booleans() | text_ascii() | sets(text_ascii()) | none(),
         log_file=booleans() | paths() | text_ascii() | none(),
@@ -216,9 +226,12 @@ class TestGetLogRecords:
         level: LogLevel | None,
         min_level: LogLevel | None,
         max_level: LogLevel | None,
-        date_or_datetime: DateOrDateTime | None,
-        min_date_or_datetime: DateOrDateTime | None,
-        max_date_or_datetime: DateOrDateTime | None,
+        date: Date | None,
+        min_date: Date | None,
+        max_date: Date | None,
+        datetime: ZonedDateTime | None,
+        min_datetime: ZonedDateTime | None,
+        max_datetime: ZonedDateTime | None,
         func_name: bool | str | None,
         extra: bool | MaybeIterable[str] | None,
         log_file: bool | PathLike | None,
@@ -232,7 +245,7 @@ class TestGetLogRecords:
         handler.setFormatter(OrjsonFormatter())
         for level_, message_, extra_ in items:
             _ = assume(set(extra_) & set(_LOG_RECORD_DEFAULT_ATTRS) == set())
-            logger.log(get_logging_level_number(level_), message_, extra=extra_)
+            logger.log(get_logging_level_number(level_), "%s", message_, extra=extra_)
         output = get_log_records(root, parallelism="threads")
         output = output.filter(
             index=index,
@@ -243,9 +256,12 @@ class TestGetLogRecords:
             level=level,
             min_level=min_level,
             max_level=max_level,
-            date_or_datetime=date_or_datetime,
-            min_date_or_datetime=min_date_or_datetime,
-            max_date_or_datetime=max_date_or_datetime,
+            date=date,
+            min_date=min_date,
+            max_date=max_date,
+            datetime=datetime,
+            min_datetime=min_datetime,
+            max_datetime=max_datetime,
             func_name=func_name,
             extra=extra,
             log_file=log_file,
@@ -270,24 +286,18 @@ class TestGetLogRecords:
             assert all(r.level >= get_logging_level_number(min_level) for r in records)
         if max_level is not None:
             assert all(r.level <= get_logging_level_number(max_level) for r in records)
-        if date_or_datetime is not None:
-            match date_or_datetime:
-                case dt.datetime() as datetime:
-                    assert all(r.datetime == datetime for r in records)
-                case dt.date() as date:
-                    assert all(r.date == date for r in records)
-        if min_date_or_datetime is not None:
-            match min_date_or_datetime:
-                case dt.datetime() as min_datetime:
-                    assert all(r.datetime >= min_datetime for r in records)
-                case dt.date() as min_date:
-                    assert all(r.date >= min_date for r in records)
-        if max_date_or_datetime is not None:
-            match max_date_or_datetime:
-                case dt.datetime() as max_datetime:
-                    assert all(r.datetime <= max_datetime for r in records)
-                case dt.date() as max_date:
-                    assert all(r.date <= max_date for r in records)
+        if date is not None:
+            assert all(r.date == date for r in records)
+        if min_date is not None:
+            assert all(r.date >= min_date for r in records)
+        if max_date is not None:
+            assert all(r.date <= max_date for r in records)
+        if datetime is not None:
+            assert all(r.datetime == datetime for r in records)
+        if min_datetime is not None:
+            assert all(r.datetime >= min_datetime for r in records)
+        if max_datetime is not None:
+            assert all(r.datetime <= max_datetime for r in records)
         if func_name is not None:
             match func_name:
                 case bool() as has_func_name:
@@ -349,8 +359,7 @@ class TestGetLogRecords:
         logger = getLogger(str(tmp_path))
         logger.addHandler(handler := FileHandler(file := tmp_path.joinpath("log")))
         handler.setFormatter(OrjsonFormatter())
-        with file.open(mode="w") as fh:
-            _ = fh.write("\n")
+        _ = file.write_text("\n")
         logger.warning("", extra={"a": 1, "b": 2, "_ignored": 3})
         result = get_log_records(tmp_path, parallelism="threads")
         assert result.path == tmp_path
@@ -371,8 +380,7 @@ class TestGetLogRecords:
     @SKIPIF_CI_AND_WINDOWS
     def test_error_file(self, *, tmp_path: Path) -> None:
         file = tmp_path.joinpath("log")
-        with file.open(mode="wb") as fh:
-            _ = fh.write(b"\x80")
+        _ = file.write_bytes(b"\x80")
         result = get_log_records(tmp_path, parallelism="threads")
         assert result.path == tmp_path
         assert result.files == [file]
@@ -398,8 +406,7 @@ class TestGetLogRecords:
 
     def test_error_deserialize_due_to_decode(self, *, tmp_path: Path) -> None:
         file = tmp_path.joinpath("log")
-        with file.open(mode="w") as fh:
-            _ = fh.write("message")
+        _ = file.write_text("message")
         result = get_log_records(tmp_path, parallelism="threads")
         assert result.path == tmp_path
         assert result.files == [file]
@@ -423,37 +430,21 @@ class TestOrjsonFormatter:
         assert record.message == "message"
         assert record.level == WARNING
         assert record.path_name == Path(__file__)
-        assert abs(record.datetime - get_now_local()) <= SECOND
+        assert abs(record.datetime - get_now()) <= SECOND
         assert record.func_name == TestOrjsonFormatter.test_main.__name__
         assert record.stack_info is None
-        assert record.extra == {"a": 1, "b": 2}
+        assert record.extra is not None
+        assert record.extra["a"] == 1
+        assert record.extra["b"] == 2
 
 
 # serialize/deserialize
 
 
 class TestSerializeAndDeserialize:
-    @given(
-        obj=make_objects(
-            dataclass_custom_equality=True,
-            dataclass_int=True,
-            dataclass_int_default=True,
-            dataclass_literal=True,
-            dataclass_literal_nullable=True,
-            dataclass_nested=True,
-            dataclass_none=True,
-            enum=True,
-            exception_class=True,
-            exception_instance=True,
-            sub_frozenset=True,
-            sub_list=True,
-            sub_set=True,
-            sub_tuple=True,
-        )
-    )
+    @given(obj=objects(all_=True, parsable=True))
     def test_all(self, *, obj: Any) -> None:
-        with assume_does_not_raise(_SerializeIntegerError):
-            ser = serialize(obj, globalns=globals())
+        ser = serialize(obj, globalns=globals())
         result = deserialize(
             ser,
             objects={
@@ -468,6 +459,8 @@ class TestSerializeAndDeserialize:
                 DataClassFutureNestedOuterFirstInner,
                 DataClassFutureNestedOuterFirstOuter,
                 DataClassFutureNone,
+                DataClassFutureTypeLiteral,
+                DataClassFutureTypeLiteralNullable,
                 SubFrozenSet,
                 SubList,
                 SubSet,
@@ -475,20 +468,19 @@ class TestSerializeAndDeserialize:
                 TruthEnum,
             },
         )
-        with assume_does_not_raise(IsEqualError):
-            assert is_equal(result, obj)
+        assert is_equal(result, obj)
 
-    @given(obj=make_objects())
+    @given(obj=objects(parsable=True))
     def test_base(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj))
         assert is_equal(result, obj)
 
-    @given(obj=make_objects(dataclass_custom_equality=True))
+    @given(obj=objects(dataclass_custom_equality=True, parsable=True))
     def test_dataclass_custom_equality(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj), objects={DataClassFutureCustomEquality})
         assert is_equal(result, obj)
 
-    @given(obj=make_objects(dataclass_default_in_init_child=True))
+    @given(obj=objects(dataclass_default_in_init_child=True, parsable=True))
     def test_dataclass_default_in_init_child_hook_in_serialize(
         self, *, obj: Any
     ) -> None:
@@ -503,7 +495,7 @@ class TestSerializeAndDeserialize:
         )
         assert is_equal(result, obj)
 
-    @given(obj=make_objects(dataclass_default_in_init_child=True))
+    @given(obj=objects(dataclass_default_in_init_child=True, parsable=True))
     def test_dataclass_default_in_init_child_hook_in_deserialize(
         self, *, obj: Any
     ) -> None:
@@ -519,28 +511,27 @@ class TestSerializeAndDeserialize:
         )
         assert is_equal(result, obj)
 
-    @given(obj=make_objects(dataclass_int=True))
+    @given(obj=objects(dataclass_int=True, parsable=True))
     def test_dataclass_int(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj), objects={DataClassFutureInt})
         assert is_equal(result, obj)
 
-    @given(obj=make_objects(dataclass_int_default=True))
+    @given(obj=objects(dataclass_int_default=True, parsable=True))
     def test_dataclass_int_default(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj), objects={DataClassFutureIntDefault})
         assert is_equal(result, obj)
 
-    @given(obj=make_objects(dataclass_literal=True))
+    @given(obj=objects(dataclass_literal=True, parsable=True))
     def test_dataclass_literal(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj), objects={DataClassFutureLiteral})
         assert is_equal(result, obj)
 
-    @given(obj=make_objects(dataclass_literal_nullable=True))
+    @given(obj=objects(dataclass_literal_nullable=True, parsable=True))
     def test_dataclass_literal_nullable(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj), objects={DataClassFutureLiteralNullable})
-        with assume_does_not_raise(IsEqualError):
-            assert is_equal(result, obj)
+        assert is_equal(result, obj)
 
-    @given(obj=make_objects(dataclass_nested=True))
+    @given(obj=objects(dataclass_nested=True, parsable=True))
     def test_dataclass_nested(self, *, obj: Any) -> None:
         ser = serialize(obj, globalns=globals())
         result = deserialize(
@@ -554,12 +545,24 @@ class TestSerializeAndDeserialize:
         )
         assert is_equal(result, obj)
 
-    @given(obj=make_objects(dataclass_none=True))
+    @given(obj=objects(dataclass_none=True, parsable=True))
     def test_dataclass_none(self, *, obj: Any) -> None:
         ser = serialize(obj, globalns=globals())
         result = deserialize(ser, objects={DataClassFutureNone})
         assert is_equal(result, obj)
 
+    @given(obj=objects(dataclass_type_literal=True, parsable=True))
+    def test_dataclass_type_literal(self, *, obj: Any) -> None:
+        result = deserialize(serialize(obj), objects={DataClassFutureTypeLiteral})
+        assert is_equal(result, obj)
+
+    @given(obj=objects(dataclass_type_literal_nullable=True, parsable=True))
+    def test_dataclass_type_literal_nullable(self, *, obj: Any) -> None:
+        result = deserialize(
+            serialize(obj), objects={DataClassFutureTypeLiteralNullable}
+        )
+        assert is_equal(result, obj)
+
     @given(obj=builds(DataClassFutureNone))
     def test_dataclass_no_objects_error(self, *, obj: DataClassFutureNone) -> None:
         ser = serialize(obj)
@@ -594,44 +597,50 @@ class TestSerializeAndDeserialize:
         )
         assert result == obj
 
-    @given(obj=make_objects(enum=True))
+    @given(obj=objects(enum=True, parsable=True))
     def test_enum(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj), objects={TruthEnum})
-        with assume_does_not_raise(IsEqualError):
-            assert is_equal(result, obj)
+        assert is_equal(result, obj)
 
-    @given(obj=make_objects(exception_class=True))
+    @given(obj=objects(exception_class=True, parsable=True))
     def test_exception_class(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj), objects={CustomError})
-        with assume_does_not_raise(IsEqualError):
-            assert is_equal(result, obj)
+        assert is_equal(result, obj)
 
-    @given(obj=make_objects(exception_instance=True))
+    @given(obj=objects(exception_instance=True, parsable=True))
     def test_exception_instance(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj), objects={CustomError})
-        with assume_does_not_raise(IsEqualError):
-            assert is_equal(result, obj)
+        assert is_equal(result, obj)
+
+    @mark.parametrize("value", [param(inf), param(-inf)])
+    def test_inf(self, *, value: float) -> None:
+        result = deserialize(serialize(value))
+        assert result == value
+
+    def test_nan(self) -> None:
+        result = deserialize(serialize(nan))
+        assert isnan(result)
 
     def test_none(self) -> None:
         result = deserialize(serialize(None))
         assert result is None
 
-    @given(obj=make_objects(sub_frozenset=True))
+    @given(obj=objects(sub_frozenset=True, parsable=True))
     def test_sub_frozenset(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj), objects={SubFrozenSet})
         assert is_equal(result, obj)
 
-    @given(obj=make_objects(sub_list=True))
+    @given(obj=objects(sub_list=True, parsable=True))
     def test_sub_list(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj), objects={SubList})
         assert is_equal(result, obj)
 
-    @given(obj=make_objects(sub_set=True))
+    @given(obj=objects(sub_set=True, parsable=True))
     def test_sub_set(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj), objects={SubSet})
         assert is_equal(result, obj)
 
-    @given(obj=make_objects(sub_tuple=True))
+    @given(obj=objects(sub_tuple=True, parsable=True))
     def test_sub_tuple(self, *, obj: Any) -> None:
         result = deserialize(serialize(obj), objects={SubTuple})
         assert is_equal(result, obj)
@@ -646,22 +655,6 @@ class TestSerializeAndDeserialize:
         )
         assert result == exp_res
 
-    @mark.parametrize(
-        ("utc", "expected"),
-        [
-            param(UTC, b'"[dt]2000-01-01T00:00:00+00:00[UTC]"'),
-            param(dt.UTC, b'"[dt]2000-01-01T00:00:00+00:00[dt.UTC]"'),
-        ],
-        ids=str,
-    )
-    def test_utc(self, *, utc: dt.tzinfo, expected: bytes) -> None:
-        datetime = dt.datetime(2000, 1, 1, tzinfo=utc)
-        ser = serialize(datetime)
-        assert ser == expected
-        result = deserialize(ser)
-        assert result == datetime
-        assert result.tzinfo is utc
-
 
 class TestSerialize:
     @given(text=text_printable())
@@ -705,14 +698,14 @@ class TestSerialize:
         class CustomError(Exception): ...
 
         result = serialize(CustomError)
-        expected = b'"[exc|TestSerialize.test_exception_class.<locals>.CustomError]"'
+        expected = b'"[Ex|TestSerialize.test_exception_class.<locals>.CustomError]"'
         assert result == expected
 
     def test_exception_instance(self) -> None:
         class CustomError(Exception): ...
 
         result = serialize(CustomError(1, 2, 3))
-        expected = b'{"[exi|TestSerialize.test_exception_instance.<locals>.CustomError]":{"[tu]":[1,2,3]}}'
+        expected = b'{"[ex|TestSerialize.test_exception_instance.<locals>.CustomError]":{"[tu]":[1,2,3]}}'
         assert result == expected
 
     @given(x=sampled_from([MIN_INT64 - 1, MAX_INT64 + 1]))
@@ -748,3 +741,13 @@ class TestObjectHookGetObject:
             match=r"Unable to find object to deserialize 'qualname' from .*",
         ):
             _ = _object_hook_get_object("qualname", objects=set())
+
+
+class TestReadAndWriteObject:
+    @given(root=temp_paths(), obj=objects(parsable=True), compress=booleans())
+    def test_main(self, *, root: Path, obj: Any, compress: bool) -> None:
+        path = root.joinpath("file.json")
+        write_object(obj, path, compress=compress)
+        assert path.is_file()
+        result = read_object(path, decompress=compress)
+        assert is_equal(result, obj)
